@@ -53,21 +53,6 @@ static GtkPrintSettings *settings = nullptr;
 const int APVLV_CURSOR_WIDTH_DEFAULT = 2;
 const int APVLV_ANNOT_UNDERLINE_HEIGHT = 10;
 
-DISPLAY_TYPE
-get_display_type_by_filename (const char *name)
-{
-  DISPLAY_TYPE type = DISPLAY_TYPE_IMAGE;
-
-  if (g_ascii_strcasecmp (name + strlen (name) - 4, ".htm") == 0
-      || g_ascii_strcasecmp (name + strlen (name) - 5, ".html") == 0
-      || g_ascii_strcasecmp (name + strlen (name) - 5, ".epub") == 0)
-    {
-      type = DISPLAY_TYPE_HTML;
-    }
-
-  return type;
-}
-
 ApvlvDoc::ApvlvDoc (ApvlvView *view, const char *zm, bool cache)
     : ApvlvCore (view)
 {
@@ -130,9 +115,9 @@ ApvlvDoc::ApvlvDoc (ApvlvView *view, const char *zm, bool cache)
       mImg[2] = nullptr;
     }
 
-  auto data_manager = webkit_website_data_manager_new (nullptr);
-  auto context
-      = webkit_web_context_new_with_website_data_manager (data_manager);
+  auto context = webkit_web_context_new_ephemeral ();
+  webkit_web_context_set_cache_model (context,
+                                      WEBKIT_CACHE_MODEL_DOCUMENT_VIEWER);
   mWeb[0] = webkit_web_view_new_with_context (context);
   auto set = webkit_web_view_get_settings (WEBKIT_WEB_VIEW (mWeb[0]));
   webkit_settings_set_default_charset (set, "UTF-8");
@@ -911,14 +896,14 @@ ApvlvDoc::loadfile (const char *filename, bool check, bool show_content)
           mCurrentCache[2] = new ApvlvDocCache (mFile);
         }
 
+      setDisplayType (mFile->get_display_type ());
+
       loadlastposition (filename);
 
       show ();
 
       setactive (true);
 
-      setDisplayType (get_display_type_by_filename (filename));
-
       mReady = true;
 
       mCurrentImage = mImg[0];
@@ -2116,12 +2101,9 @@ ApvlvDoc::webcontext_load_uri_callback (WebKitURISchemeRequest *request,
 }
 
 void
-ApvlvDoc::contentShowPage (ApvlvFileIndex *index, bool force)
+ApvlvDoc::contentShowPage (const ApvlvFileIndex &index, bool force)
 {
-  if (index == nullptr)
-    return;
-
-  if (index->type == ApvlvFileIndexType::FILE_INDEX_DIR)
+  if (index.type == ApvlvFileIndexType::FILE_INDEX_DIR)
     return;
 
   auto follow_mode = "always";
@@ -2137,26 +2119,26 @@ ApvlvDoc::contentShowPage (ApvlvFileIndex *index, bool force)
 
   else if (g_ascii_strcasecmp (follow_mode, "page") == 0)
     {
-      if (index->type == ApvlvFileIndexType::FILE_INDEX_PAGE)
+      if (index.type == ApvlvFileIndexType::FILE_INDEX_PAGE)
         {
-          if (index->page != mPagenum || index->anchor != mAnchor)
-            showpage (index->page, index->anchor);
+          if (index.page != mPagenum || index.anchor != mAnchor)
+            showpage (index.page, index.anchor);
         }
       return;
     }
 
   else if (g_ascii_strcasecmp (follow_mode, "always") == 0)
     {
-      if (index->type == ApvlvFileIndexType::FILE_INDEX_PAGE)
+      if (index.type == ApvlvFileIndexType::FILE_INDEX_PAGE)
         {
-          if (index->page != mPagenum || index->anchor != mAnchor)
-            showpage (index->page, index->anchor);
+          if (index.page != mPagenum || index.anchor != mAnchor)
+            showpage (index.page, index.anchor);
         }
       else
         {
-          if (index->path != filename ())
+          if (index.path != filename ())
             {
-              loadfile (index->path.c_str (), true, false);
+              loadfile (index.path.c_str (), true, false);
             }
         }
     }
