@@ -26,10 +26,7 @@
  */
 /* @date Created: 2008/09/30 00:00:00 Alf */
 
-#include "ApvlvView.h"
-#include "ApvlvInfo.h"
-#include "ApvlvMenuAndTool.h"
-#include "ApvlvParams.h"
+#include <filesystem>
 #include <fstream>
 #include <gdk/gdk.h>
 #include <gdk/gdkkeysyms.h>
@@ -39,6 +36,11 @@
 #include <iostream>
 #include <sstream>
 
+#include "ApvlvInfo.h"
+#include "ApvlvMenuAndTool.h"
+#include "ApvlvParams.h"
+#include "ApvlvView.h"
+
 namespace apvlv
 {
 ApvlvView::ApvlvView (ApvlvView *parent) : mCurrTabPos (-1), mCmds (this)
@@ -205,29 +207,13 @@ ApvlvView::open ()
   g_free (dirname);
 
   GtkFileFilter *filter = gtk_file_filter_new ();
-  gtk_file_filter_add_mime_type (filter, "PDF File");
-  gtk_file_filter_add_pattern (filter, "*.pdf");
-  gtk_file_filter_add_pattern (filter, "*.PDF");
-  gtk_file_filter_add_mime_type (filter, "HTML File");
-  gtk_file_filter_add_pattern (filter, "*.HTM");
-  gtk_file_filter_add_pattern (filter, "*.htm");
-  gtk_file_filter_add_pattern (filter, "*.HTML");
-  gtk_file_filter_add_pattern (filter, "*.html");
-  gtk_file_filter_add_mime_type (filter, "ePub File");
-  gtk_file_filter_add_pattern (filter, "*.EPUB");
-  gtk_file_filter_add_pattern (filter, "*.epub");
-#ifdef APVLV_WITH_DJVU
-  gtk_file_filter_add_mime_type (filter, "DJVU File");
-  gtk_file_filter_add_pattern (filter, "*.DJV");
-  gtk_file_filter_add_pattern (filter, "*.djv");
-  gtk_file_filter_add_pattern (filter, "*.DJVU");
-  gtk_file_filter_add_pattern (filter, "*.djvu");
-#endif
-#ifdef APVLV_WITH_TXT
-  gtk_file_filter_add_mime_type (filter, "TXT File");
-  gtk_file_filter_add_pattern (filter, "*.TXT");
-  gtk_file_filter_add_pattern (filter, "*.txt");
-#endif
+  auto mimes = ApvlvFile::supportMimeTypes ();
+  for (auto m = mimes.rbegin (); m != mimes.rend (); ++m)
+    {
+      gtk_file_filter_add_mime_type (filter, m->first.c_str ());
+      for (auto v = m->second.rbegin (); v != m->second.rend (); ++v)
+        gtk_file_filter_add_pattern (filter, v->c_str ());
+    }
   gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dia), filter);
 
   gint ret = gtk_dialog_run (GTK_DIALOG (dia));
@@ -504,55 +490,24 @@ ApvlvCompletion *
 ApvlvView::filecompleteinit (const char *path)
 {
   auto *comp = new ApvlvCompletion;
-  GList *list = g_list_alloc ();
-  gchar *dname, *bname;
-  const gchar *name;
+  GList *list = nullptr;
 
-  dname = g_path_get_dirname (path);
-  GDir *dir = g_dir_open ((const char *)dname, 0, nullptr);
-  if (dir != nullptr)
+  auto fspath = filesystem::path{ path };
+  auto filename = fspath.filename ();
+  auto dirname = fspath.parent_path ();
+  for (auto &entry : filesystem::directory_iterator (dirname))
     {
-      bname = g_path_get_basename (path);
-      size_t len = strlen (bname);
-      while ((name = g_dir_read_name (dir)) != nullptr)
+      auto entry_filename = entry.path ().filename ().string ();
+      if (filename.empty () || entry_filename.find (filename.string ()) == 0)
         {
-          gchar *fname
-              = g_locale_from_utf8 (name, -1, nullptr, nullptr, nullptr);
-
-          if (strcmp (bname, PATH_SEP_S) != 0)
-            {
-              if (strncmp (fname, bname, len) != 0)
-                {
-                  g_free (fname);
-                  continue;
-                }
-            }
-
-          if (strcmp (dname, ".") == 0)
-            {
-              list->data = g_strdup (fname);
-            }
-          else
-            {
-              if (dname[strlen (dname) - 1] == PATH_SEP_C)
-                {
-                  list->data = g_strjoin ("", dname, fname, nullptr);
-                }
-              else
-                {
-                  list->data = g_strjoin (PATH_SEP_S, dname, fname, nullptr);
-                }
-            }
-
-          g_free (fname);
-
-          debug ("add a item: %s", (char *)list->data);
-          comp->add_items (list);
+          auto listdata
+              = g_strdup_printf ("%s%s", entry.path ().c_str (),
+                                 entry.is_directory () ? PATH_SEP_S : "");
+          debug ("add a item: %s", listdata);
+          list = g_list_append (list, listdata);
         }
-      g_free (bname);
-      g_dir_close (dir);
     }
-  g_free (dname);
+  comp->add_items (list);
 
   g_list_free (list);
 
@@ -682,6 +637,11 @@ ApvlvView::cmd_auto (const char *ps)
   string cmd, np, argu;
   ss >> cmd >> np >> argu;
 
+  if (np.empty ())
+    {
+      return;
+    }
+
   if (np[np.length () - 1] == '\\')
     {
       np.replace (np.length () - 1, 1, 1, ' ');
